/**
 * @author Edouard DUPIN
 * 
 * @copyright 2011, Edouard DUPIN, all right reserved
 * 
 * @license BSD v3 (see license file)
 */

#ifndef __EWOL_CHECK_BOX_H__
#define __EWOL_CHECK_BOX_H__

#include <etk/Types.h>
#include <ewol/Debug.h>
#include <ewol/oObject/OObject.h>
#include <ewol/widget/Widget.h>

extern const char* const ewolEventCheckBoxClicked;

namespace ewol {
	class CheckBox : public ewol::Widget
	{
		public:
			CheckBox(void);
			CheckBox(etk::UString newLabel);
			/**
			 * @brief Get the current Object type of the EObject
			 * @note In Embended platforme, it is many time no -rtti flag, then it is not possible to use dynamic cast ==> this will replace it
			 * @param[in] objectType type description
			 * @return true if the object is compatible, otherwise false
			 */
			virtual const char * const GetObjectType(void) { return "EwolCheckBox"; };
			void Init(void);
			virtual ~CheckBox(void);
			virtual bool   CalculateMinSize(void);
			void           SetLabel(etk::UString newLabel);
			void           SetValue(bool val);
			bool           GetValue(void);
		private:
			ewol::OObject2DTextColored m_oObjectText;
			ewol::OObject2DColored     m_oObjectDecoration;
			etk::UString   m_label;
			bool           m_value;
			draw::Color       m_textColorFg;  //!< Text color
			draw::Color       m_textColorBg;  //!< Background color
		public:
			virtual void OnRegenerateDisplay(void);
			virtual void OnDraw(DrawProperty& displayProp);
		public:
			/**
			 * @brief Event on an input of this Widget
			 * @param[in] type Type of the input (ewol::INPUT_TYPE_MOUSE/ewol::INPUT_TYPE_FINGER ...)
			 * @param[in] IdInput Id of the current Input (PC : left=1, right=2, middle=3, none=0 / Tactil : first finger=1 , second=2 (only on this widget, no knowledge at ouside finger))
			 * @param[in] typeEvent ewol type of event like EVENT_INPUT_TYPE_DOWN/EVENT_INPUT_TYPE_MOVE/EVENT_INPUT_TYPE_UP/EVENT_INPUT_TYPE_SINGLE/EVENT_INPUT_TYPE_DOUBLE/...
			 * @param[in] pos Absolute position of the event
			 * @return true the event is used
			 * @return false the event is not used
			 */
			virtual bool OnEventInput(ewol::inputType_te type, int32_t IdInput, eventInputType_te typeEvent, etk::Vector2D<float>  pos);
			virtual bool OnEventKb(eventKbType_te typeEvent, uniChar_t unicodeData);
	};
	
};

#endif
